if (!lemonade && typeof (require) === 'function') {
    var lemonade = require('lemonadejs');
}

;(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
    typeof define === 'function' && define.amd ? define(factory) :
    global.Toolbar = factory();
}(this, (function () {

    const getAttributeName = function(prop) {
        if (prop.substring(0,1) === ':') {
            prop = prop.substring(1);
        } else if (prop.substring(0,3) === 'lm-') {
            prop = prop.substring(3);
        }
        return prop.toLowerCase();
    }

    const extractFromHtml =  function(element) {
        let data = [];
        // Content
        for (let i = 0; i < element.children.length; i++) {
            let e = element.children[i];
            let item = {
                type: e.getAttribute('type'),
                icon: e.getAttribute('icon') || e.getAttribute('data-icon') || '',
                title: e.textContent || e.getAttribute('title') || '',
                route: e.getAttribute('route') || e.getAttribute('href') || '',
                visible: e.getAttribute('visible') || e.getAttribute('data-visible') || '',
            }
            if (e.onclick) {
                item.onclick = e.onclick;
            }
            data.push(item);
        }

        return data;
    }

    const extract = function(children) {
        let data = [];

        if (this.tagName) {
            data = extractFromHtml(this);
            // Remove all elements
            this.textContent = '';
        } else {
            // Get data
            if (typeof(children) === 'string') {
                // Version 4
                let d = document.createElement('div');
                d.innerHTML = children;
                data = extractFromHtml(d);
            } else {
                // Version 5
                children.forEach((v) => {
                    let item = {}
                    v.props.forEach((prop) => {
                        let name = getAttributeName(prop.name);
                        if (name === 'data-icon') {
                            name = 'icon'
                        }
                        if (name === 'data-visible') {
                            name = 'visible'
                        }
                        if (name === 'href') {
                            name = 'route'
                        }
                        item[name] = prop.value;
                    });
                    if (! item.title && v.children && v.children[0]) {
                        item.title = v.children[0].props[0]?.value || '';
                    }
                    data.push(item);
                });
                // Block children
                children.length = 0;
            }
        }

        return data;
    }

    const Toolbar = function(children) {
        const self = this;

        // Toolbar data
        self.data = extract.call(this, children);

        return `<div class="lm-toolbar" data-position="{{self.position}}" data-visible="{{self.visible}}" :loop="self.data">
            <div class="lm-toolbar-item" data-selected="{{self.selected}}" data-visible="{{self.visible}}" data-gap="{{self.gap}}">
                <a href="{{self.route}}" onclick="self.onclick">
                    <img src="{{self.image}}"/><i class="material-icons material-symbols-outlined">{{self.icon}}</i><span>{{self.title}}</span>
                </a>
            </div>
        </div>`;
    }

    // Register LemonadeJS Component
    lemonade.setComponents({ Toolbar: Toolbar });
    // Register web component
    lemonade.createWebComponent('toolbar', Toolbar);

    return function(root, options, components) {
        // Render element
        if (typeof(root) === 'object') {
            // Do not accept blank options
            if (! options) {
                options = {};
            }
            // Merge component
            Toolbar.call(root);
            // Apply
            lemonade.apply(root, options);

            return options;
        } else {
            // Render element
            return Toolbar.call(this, root, components);
        }
    }
})));